/*	PPVL_get_PDS_EOL

Check for an EOL label at the end of the file's image data.


Use:

	PPVL_Parameter *
	PPVL_get_PDS_EOL
		(
		FILE			*The_File,
		PPVL_Parameter	*The_Aggregate
		)


Description:

If the EOL parameter is present in The_Aggregate, as well as all the
other parameters necessary to determine the location of the EOL label
after the end of the image data, then The_File is repositioned at the
beginning of the EOL label and it is read in as a new PPVL_Parameter
aggregate. This new aggregate is renamed "EOL" and added to the end of
The_Aggregate.

On success a pointer to the EOL aggregate is returned. On failure a 
NULL is returned and the PPVL_errno is set accordingly. An error from 
the PPVL module is passed through. Local errors are:

	PPVL_ERROR_BAD_ARGUMENT
		Either The_File or The_Aggregate are NULL.
	PPVL_ERROR_EMPTY_STATEMENT
		The_Aggregate does not have an EOL=1 parameter.
	PPVL_ERROR_ILLEGAL_SYNTAX
		The_Aggregate is missing a necessary parameter.
	PPVL_ERROR_READING_FILE
		The EOL label block could not be read from the file.


Author:

Bradford Castalia                       Castalia@Arizona.edu
Senior Systems Analyst                  (520) 621-6946
Planetary Image Research Laboratory     (520) 621-4824
Department of Planetary Sciences        FAX: (520) 621-9628
University of Arizona                   Space Sciences Bldg.
1629 E. University Blvd.                Room 429
Tucson, Arizona  85721-0092

"Build an image in your mind, fit yourself into it."
    The log of Cyradis seeress of Kell.


SCCS ID: @(#)PPVL_get_PDS_EOL.c	1.3 10/04/00
*/
static char		*SCCS_ID = "@(#)PPVL_get_PDS_EOL.c	1.3 10/04/00";

/*******************************************************************************

Copyright (C) 2000  Bradford Castalia, University of Arizona

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


Bradford Castalia                       Castalia@Arizona.edu
Senior Systems Analyst                  http://PIRL.LPL.Arizona.edu/~castalia
Planetary Image Research Laboratory     520-621-4824
Department of Planetary Sciences        FAX: 520-621-9628
University of Arizona                   Space Sciences Bldg.
1629 E. University Blvd.                Room 429
Tucson, Arizona  85721-0092

"Build an image in your mind, fit yourself into it."
    The Log of Cyradis, Seeress of Kell.

*******************************************************************************/

#include	"PPVL_selections.h"


#define	EOL_AGGREGATE_NAME	"EOL"


/*	Parameter value variables.

	The internal variables that will receive the values
	of the parameters. The initialization values flag that
	no value was received.
*/
static unsigned int
	EOL						= 0,
	Label_bytes				= 0,
	Record_bytes			= 0,
	Pixel_bytes				= 0,
	Lines					= 0,
	Columns					= 0,
	Bands					= 0,
	Line_prefix_bytes		= 0,
	Binary_header_records	= 0;

static char
	*Data_format			= NULL;


/*	Parameter selections

	This is the list of parameter names corresponding to
	the internal variables for their values.
*/
static PPVL_Parameter_Selections
		PDS_EOL_parameter_selections[] =
		{
		{"EOL",			PPVL_TYPE_INTEGER, 1, &EOL},
		{"LBLSIZE",		PPVL_TYPE_INTEGER, 1, &Label_bytes},
		{"RECSIZE",		PPVL_TYPE_INTEGER, 1, &Record_bytes},
		{"NL",			PPVL_TYPE_INTEGER, 1, &Lines},
		{"NS",			PPVL_TYPE_INTEGER, 1, &Columns},
		{"NB",			PPVL_TYPE_INTEGER, 1, &Bands},
		{"NBB",			PPVL_TYPE_INTEGER, 1, &Line_prefix_bytes},
		{"NLB",			PPVL_TYPE_INTEGER, 1, &Binary_header_records},
		{"FORMAT",		PPVL_TYPE_STRING,  1, &Data_format},
		PPVL_PARAMETER_SELECTIONS_END
		};


/*	Data_format to Pixel_bytes conversion.
*/
static String_value_mapping
	Data_format_mapping[] =
		{
		{"BYTE",	1},
		{"HALF",	2},
		PPVL_STRING_VALUE_MAPPING_END
		};


/*	What to do when an error is detected.
*/
#define ERROR(condition)        \
	{                           \
	if (condition)              \
		PPVL_errno = condition; \
	goto Error_condition;       \
	}



PPVL_Parameter *
PPVL_get_PDS_EOL
	(
	FILE			*The_File,
	PPVL_Parameter	*The_Aggregate
	)
{
PPVL_Parameter
	*EOL_aggregate;
String_value_mapping
	*string_value_mapping;
long
	EOL_offset;


if (! The_File ||
	! The_Aggregate)
	ERROR (PPVL_ERROR_BAD_ARGUMENT);


/*	Link the needed parameter values to local variables.
*/
if (PPVL_selections
		(
		The_Aggregate,
		PDS_EOL_parameter_selections
		)
	!= PPVL_SUCCESS)
	ERROR (0);

/*	Confirm that everything needed is available.
*/
if (! EOL)
	ERROR (PPVL_ERROR_EMPTY_STATEMENT);

if (! Label_bytes ||
	! Lines ||
	! Columns ||
	! Bands ||
	! Data_format)
	ERROR (PPVL_ERROR_ILLEGAL_SYNTAX);

for (string_value_mapping = Data_format_mapping;
	 string_value_mapping->string;
	 string_value_mapping++)
	{
	if (! strcmp (Data_format, string_value_mapping->string))
		{
		Pixel_bytes = string_value_mapping->value;
		break;
		}
	}
if (! Pixel_bytes)
	ERROR (PPVL_ERROR_ILLEGAL_SYNTAX);

if (Binary_header_records &&
	! Record_bytes)
	ERROR (PPVL_ERROR_ILLEGAL_SYNTAX);


/*	Position the file at the EOL label.
*/
EOL_offset =
	Label_bytes +
	(Pixel_bytes * Lines * (Line_prefix_bytes + Columns) * Bands) +
	(Record_bytes * Binary_header_records);

if (fseek (The_File, EOL_offset, SEEK_SET))
	ERROR (PPVL_ERROR_READING_FILE);


/*	Read the EOL label into a new PPVL parameter aggregate.
*/
if (! (EOL_aggregate = PPVL_read_aggregate
		(
		The_File,
		PPVL_NO_READ_LIMIT,
		NULL
		)))
	ERROR (0);


/*	Change the default name of the EOL_aggregate - "The Container" -
	to a more appropriate (but no longer) name.
*/
strcpy (EOL_aggregate->name, EOL_AGGREGATE_NAME);


/*	Add the EOL_aggregate to The_Aggregate.
*/
if (! PPVL_add_parameter
		(
		The_Aggregate,
		EOL_aggregate
		))
	ERROR (0);

return (EOL_aggregate);


Error_condition:


return (NULL);
}

